% nimasImagesfromMpA() - Generates blobs or voxels inside the brain space
%                        using images from Nima's Measure Projection. Input
%                        must be single or multiple cells containing 1x3
%                        xyz coordinates of dipole locations. It applies
%                        3-D Gaussian blurr kernel to dipole locations to
%                        make them into probabilistic dipole density, and
%                        visualize it with either blobs or voxels. The
%                        preset voxels size is 8x8x8 mm (determined by
%                        headGrid.mat). This function requires several
%                        files that were stripped fromo Measure Projection
%                        Toolbox by Makoto to make it available as a
%                        stand-alone function.
%
% Usage:
%
% nimasImagesfromMpA(dipoleLocations, varargin)
%
% Input:
%
% dipoleLocations - {1 x n} cells, each containing [m x 3] which is
%                   EEG.dipfit.model.posxzy.
%
% Optional Input as varargin:
%
%     'voxelSize' - spatial resolution in mm. [default: 4]
%     'FWHM' - full-width half-maximum of 3-D Gaussian kernel in mm. [default: 20]
%     'numGaussTrunc' - number of sigma to truncate the Gaussian. [default: 2]
%     'blobOrVoxel' - selection of the type of visualization. [default: 'blob']
%     'blobAlpha' - transparency of the blob/voxel. [default: 0.3]
%     'edgeAlpha' - this applies to voxel only. [default: 0.6]
%     'edgeColor' - this applies to voxel only. [default: 0.5 0.5 0.5]
%     'mainLightIntensity' - brightness of the main light. [default: 1]     
%     'secondaryLightIntensity' - same as above. [default: 1]
%     'blobColor' - this needs to be [n x 3] where n is the number of cells 
%                   in the input 'dipoleLocations' and 3 is RGB values.
%                   If not specified, 13 W3C colors will be used by default.
%                   See below for the list of the 13 colors.
%
%                   colors{1,1}  = [1 1 1];            % White (excluded)
%                   colors{2,1}  = [1 1 0];            % Yellow
%                   colors{3,1}  = [1 0 1];            % Fuchsia
%                   colors{4,1}  = [1 0 0];            % Red
%                   colors{5,1}  = [0.75  0.75  0.75]; % Silver
%                   colors{6,1}  = [0.5 0.5 0.5];      % Gray (excluded)
%                   colors{7,1}  = [0.5 0.5 0];        % Olive
%                   colors{8,1}  = [0.5 0 0.5];        % Purple
%                   colors{9,1}  = [0.5 0 0];          % Maroon
%                   colors{10,1} = [0 1 1];            % Aqua
%                   colors{11,1} = [0 1 0];            % Lime
%                   colors{12,1} = [0 0.5 0.5];        % Teal
%                   colors{13,1} = [0 0.5 0];          % Green
%                   colors{14,1} = [0 0 1];            % Blue
%                   colors{15,1} = [0 0 0.5];          % Navy
%                   colors{16,1} = [0 0 0];            % Black (excluded)

% Copyright (C) 2018, Makoto Miyakoshi and Nima Bigdely-Shamlo, SCCN, INC, UCSD.
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1.07  USA

% History
% 12/05/2018 Makoto. Created.

function nimasImagesfromMpA(dipoleLocations, varargin)

% Remove or add optionalInputs.
if isempty(varargin{end})
    varargin = varargin(1:end-1);
else
    vararginContents = varargin{end};
    varargin = varargin(1:end-1);
    varargin = [varargin vararginContents];
end

% Prepare inputs.
inputOptions = finputcheck(varargin, ...
    {'voxelSize'               'real'   [2 8] 4;...
     'FWHM'                    'real'   []    20;...
     'numGaussTrunc'           'real'   []    3;...
     'blobOrVoxel'             'real'   [1 2] 1;...
     'blobAlpha'               'real'   []    0.30;...
     'edgeAlpha'               'real'   []    0.60;...
     'edgeColor'               'real'   []    [0.5 0.5 0.5];...
     'blobColor'               'real'   []    [];...
     'mainLightIntensity'      'real'   [0 1] 1;...
     'secondaryLightIntensity' 'real'   [0 1] 1;...
    });



%%%%%%%%%%%%%%%%%%%%%%%
%%% Prepare colors. %%%
%%%%%%%%%%%%%%%%%%%%%%%
if isempty(inputOptions.blobColor)
    
    userSpecifiedBlobColorFlag = 0;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% Set 12 blob colors.%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%
    % 16 color names officially supported by W3C specification for HTML
    colors{1,1}  = [1 1 1];            % White (excluded)
    colors{2,1}  = [1 1 0];            % Yellow
    colors{3,1}  = [1 0 1];            % Fuchsia
    colors{4,1}  = [1 0 0];            % Red
    colors{5,1}  = [0.75  0.75  0.75]; % Silver
    colors{6,1}  = [0.5 0.5 0.5];      % Gray (excluded)
    colors{7,1}  = [0.5 0.5 0];        % Olive
    colors{8,1}  = [0.5 0 0.5];        % Purple
    colors{9,1}  = [0.5 0 0];          % Maroon
    colors{10,1} = [0 1 1];            % Aqua
    colors{11,1} = [0 1 0];            % Lime
    colors{12,1} = [0 0.5 0.5];        % Teal
    colors{13,1} = [0 0.5 0];          % Green
    colors{14,1} = [0 0 1];            % Blue
    colors{15,1} = [0 0 0.5];          % Navy
    colors{16,1} = [0 0 0];            % Black (excluded)
    
    % Silver is twice brighter because the color may be used for background
    colors{5,1} = [0.875 0.875 0.875];
    
    % Exclusing:White, Black, Gray. Choosing and sorting the remaining 13 colors: Red, Blue, Green, Yellow, Fuchsia, Lime, Aqua, Maroon, Olive, Purple, Teal, Navy, and Silver.
    inputOptions.blobColor = colors([4 13 14 2 3 11 10 9 7 8 12 15 5]);
else
    
    userSpecifiedBlobColorFlag = 1;
    
    % Convert nx3 matrix to cell array.
    colors = cell(size(inputOptions.blobColor, 1),1);
    for colorIdx = 1:size(inputOptions.blobColor, 1) 
        colors{colorIdx,1} = inputOptions.blobColor(colorIdx,:);
    end
    inputOptions.blobColor = colors;
    
    if length(dipoleLocations) == size(inputOptions.blobColor, 1)
        disp('User defined color will be used.')
    else
        error('The number of colors should be the same as the number of blobs to plot')
    end
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Prepare 3-D Gaussian kernel. %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load headGrid.
switch inputOptions.voxelSize
    case 2
        load('headGrid_2mm.mat');
    case 3
        load('headGrid_3mm.mat');
    case 4
        load('headGrid_4mm.mat');
    case 5
        load('headGrid_5mm.mat');
    case 6
        load('headGrid_6mm.mat');
    case 7
        load('headGrid_7mm.mat');
    case 8
        load('headGrid_8mm.mat');
end

% Perepare the 3-D Gaussian kernel.
sigmaInGaussian = inputOptions.FWHM/2.355; % This calculates sigma in Gaussian equation.
parametersForProjection = projectionParameter(sigmaInGaussian);
parametersForProjection.numberOfStandardDeviationsToTruncatedGaussaian = inputOptions.numGaussTrunc;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Open the main figure. %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure
plot_dipplot_with_cortex;
plotOptions   = {};
regionOptions = {};


%%%%%%%%%%%%%%%%%%%%%%%
%%% Start the loop. %%%
%%%%%%%%%%%%%%%%%%%%%%%
for numBlobIdx = 1:length(dipoleLocations)
    
    % Obtain the current dipole cluster.
    dipoleInfo.location = dipoleLocations{numBlobIdx};
    
    % Transform dipole locations into dipole density.
    [projectionMatrix, totalDipoleDenisty, dipoleDensity] = getProjectionMatrix(dipoleInfo, headGrid, parametersForProjection);
    
    % Obtain inside-brain inclusive mask.
    insideBrainInclusiveMaskIdx = find(vec(headGrid.insideBrainCube));
    
    % Create a mask for inside-brain inclusive voxel mask on 27x23x23 tensor.
    emptyBrainToBePopulated1D = vec(zeros(headGrid.cubeSize));
    emptyBrainToBePopulated1D(insideBrainInclusiveMaskIdx) = sum(dipoleDensity,1);
    sumDipoleDensity = reshape(emptyBrainToBePopulated1D, headGrid.cubeSize);

    % Determine the current color index.
    if userSpecifiedBlobColorFlag == 0;
        colorIdx = mod(numBlobIdx, length(inputOptions.blobColor));
        if colorIdx == 0
            colorIdx = length(inputOptions.blobColor);
        end
    else
        colorIdx = numBlobIdx;
    end
    
    % Show blob or voxel.
    switch inputOptions.blobOrVoxel
        case 1
            surfaceOptions = {'FaceAlpha' inputOptions.blobAlpha};
            plot_head_surface(headGrid, logical(sumDipoleDensity),...
                'surfaceColor', inputOptions.blobColor{colorIdx},...
                'surfaceOptions', surfaceOptions,...
                'mainLightIntensity', inputOptions.mainLightIntensity,...
                'secondaryLightIntensity', inputOptions.secondaryLightIntensity,...
                plotOptions{:});
            
        case 2
            plot_head_region(headGrid, sumDipoleDensity,...
                'regionColor',  inputOptions.blobColor{colorIdx},...
                'projectionAlpha', inputOptions.blobAlpha,...
                'edgeAlpha', inputOptions.edgeAlpha,...
                'edgeColor', inputOptions.edgeColor,...
                'regionOptions',  regionOptions);
    end
end